(***************************************************************************
****************************************************************************
**                                                                        **
**                XXXX     X    X         X XXXXXX X                      **
**               X    X   X X   X    X    X X      X   X                  **
**               X       X   X  X   X X   X X      X  X                   **
**               X   XX XXXXXXX X  X   X  X XXXX   X X                    **
**               X    X X     X X X     X X X      XX                     **
**               X    X X     X XX       XX X      X                      **
**                XXXX  X     X X         X XXXXXX XXXXXX                 **
**                                                                        **
**                           BIALYSTOK 2020                               **
**                                                                        **
**             UDPTester - program demostracyjny filozofii sieci          **
**                      Wersja z dnia : 05.10.2020                        **
**                     Napisal : Andrzej Pawluczuk                        **
**                                                                        **
****************************************************************************
***************************************************************************)

unit udptesterunit ;

{$mode objfpc}{$H+}

interface

uses
  Classes, SysUtils, FileUtil, Forms, Controls, Graphics, Dialogs, StdCtrls,
  ExtCtrls, ComCtrls, blcksock;

type

  { TUDPTesterForm }

  TUDPTesterForm = class ( TForm )
    MyIPAddressLabel         : TLabel ;
    DestIPLabel              : TLabel ;
    DestPortLabel            : TLabel ;
    MyPortNoLabel            : TLabel ;
    DataLabel                : TLabel ;
    ClearButton              : TButton ;
    ExitButton               : TButton ;
    ChangeMyPortButton       : TButton ;
    SendDataButton           : TButton ;
    DestIPEdit               : TEdit ;
    DestPortEdit             : TEdit ;
    MyPortEdit               : TEdit ;
    MyIPEdit                 : TEdit ;
    DataEdit                 : TEdit ;
    StatusBar                : TStatusBar ;
    TextMemo                 : TMemo ;
    GeneralTimer             : TTimer ;
    procedure ClearButtonClick ( Sender : TObject ) ;
    procedure ExitButtonClick ( Sender : TObject ) ;
    procedure FormCreate ( Sender : TObject ) ;
    procedure FormDestroy ( Sender : TObject ) ;
    procedure GeneralTimerTimer ( Sender : TObject ) ;
    procedure ChangeMyPortButtonClick ( Sender : TObject ) ;
    procedure SendDataButtonClick ( Sender : TObject ) ;
  private
    UDPSocket                : TUDPBlockSocket ;
    function AcceptOperation : boolean ;
  public
    { public declarations }
  end ;

const
  DefaultUDPListenPortNo         = 1000 ;
  MachinePanelIndex              = 0 ;
  IPAddressPanelIndex            = 1 ;
  PortNoPanelIndex               = 2 ;
  InputDataSymbol                = '> ' ;
  OutputDataSymbol               = '< ' ;
  NetParamSeparator              = ':' ;
  DataSeparator                  = ' = ' ;

var
  UDPTesterForm                  : TUDPTesterForm ;


implementation
{$R *.lfm}

{ TUDPTesterForm }


function TUDPTesterForm . AcceptOperation : boolean ;
begin (* TUDPTesterForm . AcceptOperation *)
  if MessageDlg ( 'Potwierdzenie' , 'Czy potwierdzasz operację?' , mtConfirmation , [ mbYes , mbNo ] , 0 ) = mrYes then
    AcceptOperation := true
  else
    AcceptOperation := false ;
end (* TUDPTesterForm . AcceptOperation *) ;


procedure TUDPTesterForm . FormCreate ( Sender : TObject ) ;
  var
    MachineName         : String ;
    IPAddress           : String ;
begin (* TUDPTesterForm . FormCreate *)
  UDPSocket := TUDPBlockSocket . Create ;
  UDPSocket . Family := SF_IP4 ;
  TextMemo . Clear ;
  MyPortEdit . Text := IntToStr ( DefaultUDPListenPortNo ) ;
  MachineName := UDPSocket . LocalName ;
  IPAddress := UDPSocket . ResolveName ( MachineName ) ;
  StatusBar . Panels . Items [ MachinePanelIndex ] . Text := 'Komputer: ' + MachineName ;
  StatusBar . Panels . Items [ IPAddressPanelIndex ] . Text := 'IP: ' + IPAddress ;
  MyIPEdit . Text := IPAddress ;
  ChangeMyPortButtonClick ( Sender ) ;
  GeneralTimer . Enabled := true ;
end (* TUDPTesterForm . FormCreate *) ;


procedure TUDPTesterForm . FormDestroy ( Sender : TObject ) ;
begin (* TUDPTesterForm . FormDestroy *)
  UDPSocket . Destroy ;
end (* TUDPTesterForm . FormDestroy *) ;


procedure TUDPTesterForm . GeneralTimerTimer ( Sender : TObject ) ;
  var
    MemoMessage         : String ;
    Data                : String ;
    RemIPAdr            : String ;
    RemPort             : integer ;
begin (* TUDPTesterForm . GeneralTimerTimer *)
  GeneralTimer . Enabled := false ;
  Data := UDPSocket . RecvPacket ( 0 ) ;
  if length ( Data ) <> 0 then
  begin (* 1 *)
    RemIPAdr := UDPSocket . GetRemoteSinIP ;
    RemPort := UDPSocket . GetRemoteSinPort ;
    MemoMessage := InputDataSymbol + RemIPAdr + NetParamSeparator + IntToStr ( RemPort ) +
                   DataSeparator + Data ;
    TextMemo . Lines . Add ( MemoMessage ) ;
  end (* 1 *) ;
  GeneralTimer . Enabled := true ;
end (* TUDPTesterForm . GeneralTimerTimer *) ;


function ValidPortNo ( Port : String ) : boolean ;
  var
    PortNumer           : integer ;

  begin (* ValidPortNo *)
    if not TryStrToInt ( Port , PortNumer ) then
    begin (* 1 *)
      ShowMessage ( 'Niepoprawny numer portu.' ) ;
      ValidPortNo := false ;
      exit ;
    end (* 1 *) ;
    if ( PortNumer <= 0 ) or ( PortNumer > 65535 ) then
    begin (* 1 *)
      ShowMessage ( 'Numer portu spaza zakresu' ) ;
      ValidPortNo := false ;
      exit ;
    end (* 1 *) ;
    ValidPortNo := true ;
  end (* ValidPortNo *) ;


procedure TUDPTesterForm.ChangeMyPortButtonClick ( Sender : TObject ) ;
  var
    PortNo              : String ;

begin (* TUDPTesterForm.ChangeMyPortButtonClick *)
  PortNo := Trim ( MyPortEdit . Text ) ;
  if not ValidPortNo ( PortNo ) then
    exit ;
  UDPSocket . Destroy ;
  UDPSocket := TUDPBlockSocket . Create ;
  UDPSocket . Bind ( '0.0.0.0' , MyPortEdit . Text ) ;
  UDPSocket . Listen ;
  StatusBar . Panels . Items [ PortNoPanelIndex ] . Text := 'Port: ' + MyPortEdit . Text ;
end (* TUDPTesterForm.ChangeMyPortButtonClick *) ;


procedure TUDPTesterForm . SendDataButtonClick ( Sender : TObject ) ;
  var
    MemoMessage         : String ;
    DestinationAddress  : String ;
    DestinationPort     : String ;
    DataText            : String ;

  function ValidAddress ( Address : String ) : boolean ;
    var
      Index               : cardinal ;
      StrSize             : cardinal ;

    function ExtractNumber : boolean ;
      var
        SubStr            : string ;
        Numb              : integer ;
        Ch                : char ;
        LoopExit          : boolean ;

      begin (* ExtractNumber *)
        SubStr := '' ;
        LoopExit := false ;
        repeat
          if Index > StrSize then
          begin (* 1 *)
            LoopExit := true ;
          end (* 1 *)
          else
          begin (* 1 *)
            Ch := Address [ Index ] ;
            Index := Index + 1 ;
            if Ch = '.' then
              LoopExit := true
            else
              SubStr := SubStr + Ch ;
          end (* 1 *)
        until LoopExit ;
        if Length ( SubStr ) <= 0 then
        begin (* 1 *)
          ExtractNumber := false ;
          exit ;
        end (* 1 *) ;
        if not TryStrToInt ( SubStr , Numb ) then
        begin (* 1 *)
          ExtractNumber := false ;
          exit ;
        end (* 1 *) ;
        if Numb > 255 then
          ExtractNumber := false
        else
          ExtractNumber := true ;
      end (* ExtractNumber *) ;

    begin (* ValidAddress *)
      StrSize := Length ( Address ) ;
      Index := 1 ;
      if not ExtractNumber then
      begin (* 1 *)
        ValidAddress := false ;
        exit ;
      end (* 1 *) ;
      if not ExtractNumber then
      begin (* 1 *)
        ValidAddress := false ;
        exit ;
      end (* 1 *) ;
      if not ExtractNumber then
      begin (* 1 *)
        ValidAddress := false ;
        exit ;
      end (* 1 *) ;
      if not ExtractNumber then
      begin (* 1 *)
        ValidAddress := false ;
        exit ;
      end (* 1 *) ;
      ValidAddress := true ;
    end (* ValidAddress *) ;

begin (* TUDPTesterForm . SendDataButtonClick *)
  DestinationAddress := Trim ( DestIPEdit . Text ) ;
  if Length ( DestinationAddress ) <= 0 then
  begin (* 1 *)
    DestinationAddress := '127.0.0.1' ;
    DestIPEdit . Text := DestinationAddress ;
  end (* 1 *) ;
  if not ValidAddress ( DestinationAddress ) then
  begin (* 1 *)
    ShowMessage ( 'Niepoprawny adres IP.' ) ;
    exit ;
  end (* 1 *) ;
  DestinationPort := Trim ( DestPortEdit . Text ) ;
  if not ValidPortNo ( DestinationPort ) then
    exit ;
  DataText := DataEdit . Text ;
  if Length ( DataText ) <= 0 then
    DataText := '<puste>' ;
  UDPSocket . connect ( DestinationAddress , DestinationPort ) ;
  UDPSocket . SendString ( DataText ) ;
  MemoMessage := OutputDataSymbol + DestinationAddress + NetParamSeparator + DestinationPort +
                 DataSeparator + DataText ;
  TextMemo . Lines . Add ( MemoMessage ) ;
end (* TUDPTesterForm . SendDataButtonClick *) ;


procedure TUDPTesterForm . ExitButtonClick ( Sender : TObject ) ;
begin (* TUDPTesterForm . ExitButtonClick *)
  if AcceptOperation then
    Application . Terminate ;
end (* TUDPTesterForm . ExitButtonClick *) ;


procedure TUDPTesterForm.ClearButtonClick ( Sender : TObject ) ;
begin (* TUDPTesterForm.ClearButtonClick *)
  if AcceptOperation then
    TextMemo . Clear ;
end (* TUDPTesterForm.ClearButtonClick *) ;



end.

